@@ -29,6 +29,7 @@ import threading
 import base64
 import ConfigParser
 import itertools
+import shutil
 import logging
 
 import gtk
@@ -97,6 +98,7 @@ class Dialcentral(object):
 		self._alarmHandler = None
 		self._ledHandler = None
 		self._originalCurrentLabels = []
+		self._fsContactsPath = os.path.join(constants._data_path_, "contacts")
 
 		for path in self._glade_files:
 			if os.path.isfile(path):
@@ -111,6 +113,7 @@ class Dialcentral(object):
 		self._notebook = self._widgetTree.get_widget("notebook")
 		self._errorDisplay = gtk_toolbox.ErrorDisplay(self._widgetTree)
 		self._credentialsDialog = gtk_toolbox.LoginWindow(self._widgetTree)
+		self._smsEntryWindow = None
 
 		self._isFullScreen = False
 		self._app = hildonize.get_app_class()()
@@ -139,7 +142,6 @@ class Dialcentral(object):
 			"addressbookSelectButton",
 			"sendSmsButton",
 			"dialButton",
-			"cancelSmsButton",
 			"callbackSelectButton",
 			"minutesEntryButton",
 			"clearcookies",
@@ -158,7 +160,7 @@ class Dialcentral(object):
 			button.connect("clicked", self._on_clearcookies_clicked)
 			menu.append(button)
 
-			button= gtk.Button("Refresh")
+			button = gtk.Button("Refresh")
 			button.connect("clicked", self._on_menu_refresh)
 			menu.append(button)
 
@@ -276,6 +278,7 @@ class Dialcentral(object):
 			import gv_views
 			from backends import merge_backend
 
+			self._smsEntryWindow = gv_views.SmsEntryWindow(self._widgetTree, self._window)
 			try:
 				os.makedirs(constants._data_path_)
 			except OSError, e:
@@ -313,13 +316,14 @@ class Dialcentral(object):
 					),
 				})
 
-			fsContactsPath = os.path.join(constants._data_path_, "contacts")
-			fileBackend = file_backend.FilesystemAddressBookFactory(fsContactsPath)
+			fileBackend = file_backend.FilesystemAddressBookFactory(self._fsContactsPath)
 
-			self._dialpads[self.GV_BACKEND].number_selected = self._select_action
-			self._historyViews[self.GV_BACKEND].number_selected = self._select_action
-			self._messagesViews[self.GV_BACKEND].number_selected = self._select_action
-			self._contactsViews[self.GV_BACKEND].number_selected = self._select_action
+			self._smsEntryWindow.send_sms = self._on_sms_clicked
+			self._smsEntryWindow.dial = self._on_dial_clicked
+			self._dialpads[self.GV_BACKEND].add_contact = self._add_contact
+			self._historyViews[self.GV_BACKEND].add_contact = self._add_contact
+			self._messagesViews[self.GV_BACKEND].add_contact = self._add_contact
+			self._contactsViews[self.GV_BACKEND].add_contact = self._add_contact
 
 			addressBooks = [
 				self._phoneBackends[self.GV_BACKEND],
@@ -485,14 +489,8 @@ class Dialcentral(object):
 
 		return loggedIn, serviceId
 
-	def _select_action(self, action, number, message):
-		self.refresh_session()
-		if action == "dial":
-			self._on_dial_clicked(number)
-		elif action == "sms":
-			self._on_sms_clicked(number, message)
-		else:
-			assert False, "Unknown action: %s" % action
+	def _add_contact(self, *args, **kwds):
+		self._smsEntryWindow.add_contact(*args, **kwds)
 
 	def _change_loggedin_status(self, newStatus):
 		oldStatus = self._selectedBackendId
@@ -753,6 +751,8 @@ class Dialcentral(object):
 				self._window.destroy()
 			elif event.keyval == gtk.keysyms.r and event.get_state() & gtk.gdk.CONTROL_MASK:
 				self._refresh_active_tab()
+			elif event.keyval == gtk.keysyms.i and event.get_state() & gtk.gdk.CONTROL_MASK:
+				self._import_contacts()
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
@@ -817,10 +817,11 @@ class Dialcentral(object):
 			self._errorDisplay.push_exception()
 		return False
 
-	def _on_sms_clicked(self, number, message):
+	def _on_sms_clicked(self, numbers, message):
 		try:
-			assert number, "No number specified"
+			assert numbers, "No number specified"
 			assert message, "Empty message"
+			self.refresh_session()
 			try:
 				loggedIn = self._phoneBackends[self._selectedBackendId].is_authed()
 			except Exception, e:
@@ -836,9 +837,9 @@ class Dialcentral(object):
 
 			dialed = False
 			try:
-				self._phoneBackends[self._selectedBackendId].send_sms(number, message)
-				hildonize.show_information_banner(self._window, "Sending to %s" % number)
-				_moduleLogger.info("Sending SMS to %s" % number)
+				self._phoneBackends[self._selectedBackendId].send_sms(numbers, message)
+				hildonize.show_information_banner(self._window, "Sending to %s" % ", ".join(numbers))
+				_moduleLogger.info("Sending SMS to %r" % numbers)
 				dialed = True
 			except Exception, e:
 				self._errorDisplay.push_exception()
@@ -851,6 +852,7 @@ class Dialcentral(object):
 	def _on_dial_clicked(self, number):
 		try:
 			assert number, "No number to call"
+			self.refresh_session()
 			try:
 				loggedIn = self._phoneBackends[self._selectedBackendId].is_authed()
 			except Exception, e:
@@ -879,6 +881,24 @@ class Dialcentral(object):
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
+	def _import_contacts(self):
+		csvFilter = gtk.FileFilter()
+		csvFilter.set_name("Contacts")
+		csvFilter.add_pattern("*.csv")
+		importFileChooser = gtk.FileChooserDialog(
+			title="Contacts",
+			parent=self._window,
+		)
+		importFileChooser.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
+		importFileChooser.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
+
+		importFileChooser.set_property("filter", csvFilter)
+		userResponse = importFileChooser.run()
+		importFileChooser.hide()
+		if userResponse == gtk.RESPONSE_OK:
+			filename = importFileChooser.get_filename()
+			shutil.copy2(filename, self._fsContactsPath)
+
 	def _on_menu_refresh(self, *args):
 		try:
 			self._refresh_active_tab()
