@@ -19,14 +19,14 @@ License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 @todo Collapse voicemails
-@todo Alternate UI for dialogs (stackables)
 """
 
 from __future__ import with_statement
 
+import re
 import ConfigParser
-import logging
 import itertools
+import logging
 
 import gobject
 import pango
@@ -47,13 +47,56 @@ def make_ugly(prettynumber):
 	characters
 
 	>>> make_ugly("+012-(345)-678-90")
-	'01234567890'
+	'+01234567890'
 	"""
-	import re
-	uglynumber = re.sub('\D', '', prettynumber)
+	return normalize_number(prettynumber)
+
+
+def normalize_number(prettynumber):
+	"""
+	function to take a phone number and strip out all non-numeric
+	characters
+
+	>>> normalize_number("+012-(345)-678-90")
+	'+01234567890'
+	>>> normalize_number("1-(345)-678-9000")
+	'13456789000'
+	>>> normalize_number("+1-(345)-678-9000")
+	'+13456789000'
+	"""
+	uglynumber = re.sub('[^0-9+]', '', prettynumber)
+
 	return uglynumber
 
 
+def _make_pretty_with_areacodde(phonenumber):
+	prettynumber = "(%s)" % (phonenumber[0:3], )
+	if 3 < len(phonenumber):
+		prettynumber += " %s" % (phonenumber[3:6], )
+		if 6 < len(phonenumber):
+			prettynumber += "-%s" % (phonenumber[6:], )
+	return prettynumber
+
+
+def _make_pretty_local(phonenumber):
+	prettynumber = "%s" % (phonenumber[0:3], )
+	if 3 < len(phonenumber):
+		prettynumber += "-%s" % (phonenumber[3:], )
+	return prettynumber
+
+
+def _make_pretty_international(phonenumber):
+	prettynumber = phonenumber
+	if phonenumber.startswith("0"):
+		prettynumber = "+%s " % (phonenumber[0:3], )
+		if 3 < len(phonenumber):
+			prettynumber += _make_pretty_with_areacodde(phonenumber[3:])
+	if phonenumber.startswith("1"):
+		prettynumber = "1 "
+		prettynumber += _make_pretty_with_areacodde(phonenumber[1:])
+	return prettynumber
+
+
 def make_pretty(phonenumber):
 	"""
 	Function to take a phone number and return the pretty version
@@ -71,37 +114,38 @@ def make_pretty(phonenumber):
 	>>> make_pretty("1234567")
 	'123-4567'
 	>>> make_pretty("2345678901")
-	'(234)-567-8901'
+	'+1 (234) 567-8901'
 	>>> make_pretty("12345678901")
-	'1 (234)-567-8901'
+	'+1 (234) 567-8901'
 	>>> make_pretty("01234567890")
-	'+012-(345)-678-90'
+	'+012 (345) 678-90'
+	>>> make_pretty("+01234567890")
+	'+012 (345) 678-90'
+	>>> make_pretty("+12")
+	'+1 (2)'
+	>>> make_pretty("+123")
+	'+1 (23)'
+	>>> make_pretty("+1234")
+	'+1 (234)'
 	"""
 	if phonenumber is None or phonenumber is "":
 		return ""
 
-	phonenumber = make_ugly(phonenumber)
+	phonenumber = normalize_number(phonenumber)
 
-	if len(phonenumber) < 3:
-		return phonenumber
-
-	if phonenumber[0] == "0":
-		prettynumber = ""
-		prettynumber += "+%s" % phonenumber[0:3]
-		if 3 < len(phonenumber):
-			prettynumber += "-(%s)" % phonenumber[3:6]
-			if 6 < len(phonenumber):
-				prettynumber += "-%s" % phonenumber[6:9]
-				if 9 < len(phonenumber):
-					prettynumber += "-%s" % phonenumber[9:]
-		return prettynumber
-	elif len(phonenumber) <= 7:
-		prettynumber = "%s-%s" % (phonenumber[0:3], phonenumber[3:])
-	elif len(phonenumber) > 8 and phonenumber[0] == "1":
-		prettynumber = "1 (%s)-%s-%s" % (phonenumber[1:4], phonenumber[4:7], phonenumber[7:])
-	elif len(phonenumber) > 7:
-		prettynumber = "(%s)-%s-%s" % (phonenumber[0:3], phonenumber[3:6], phonenumber[6:])
-	return prettynumber
+	if phonenumber[0] == "+":
+		prettynumber = _make_pretty_international(phonenumber[1:])
+		if not prettynumber.startswith("+"):
+			prettynumber = "+"+prettynumber
+	elif 8 < len(phonenumber) and phonenumber[0] in ("0", "1"):
+		prettynumber = _make_pretty_international(phonenumber)
+	elif 7 < len(phonenumber):
+		prettynumber = _make_pretty_with_areacodde(phonenumber)
+	elif 3 < len(phonenumber):
+		prettynumber = _make_pretty_local(phonenumber)
+	else:
+		prettynumber = phonenumber
+	return prettynumber.strip()
 
 
 def abbrev_relative_date(date):
@@ -165,206 +209,305 @@ def collapse_message(message, maxCharsPerLine, maxLines):
 	return "\n".join(_collapse_message(messageLines, maxCharsPerLine, maxLines))
 
 
-class SmsEntryDialog(object):
-	"""
-	@todo Add multi-SMS messages like GoogleVoice
-	"""
-
-	ACTION_CANCEL = "cancel"
-	ACTION_DIAL = "dial"
-	ACTION_SEND_SMS = "sms"
+class SmsEntryWindow(object):
 
 	MAX_CHAR = 160
 
-	def __init__(self, widgetTree):
+	def __init__(self, widgetTree, parent):
 		self._clipboard = gtk.clipboard_get()
 		self._widgetTree = widgetTree
-		self._dialog = self._widgetTree.get_widget("smsDialog")
+		self._window = self._widgetTree.get_widget("smsWindow")
+		self._window.connect("delete-event", self._on_delete)
+		self._window.connect("key-press-event", self._on_key_press)
+		self._window.connect("window-state-event", self._on_window_state_change)
+		self._isFullScreen = False
+		self._parent = parent
 
 		self._smsButton = self._widgetTree.get_widget("sendSmsButton")
 		self._smsButton.connect("clicked", self._on_send)
 		self._dialButton = self._widgetTree.get_widget("dialButton")
 		self._dialButton.connect("clicked", self._on_dial)
-		self._cancelButton = self._widgetTree.get_widget("cancelSmsButton")
-		self._cancelButton.connect("clicked", self._on_cancel)
 
 		self._letterCountLabel = self._widgetTree.get_widget("smsLetterCount")
 
 		self._messagemodel = gtk.ListStore(gobject.TYPE_STRING)
 		self._messagesView = self._widgetTree.get_widget("smsMessages")
 
+		textrenderer = gtk.CellRendererText()
+		textrenderer.set_property("wrap-mode", pango.WRAP_WORD)
+		textrenderer.set_property("wrap-width", 450)
+		messageColumn = gtk.TreeViewColumn("")
+		messageColumn.pack_start(textrenderer, expand=True)
+		messageColumn.add_attribute(textrenderer, "markup", 0)
+		messageColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+		self._messagesView.append_column(messageColumn)
+		self._messagesView.set_headers_visible(False)
+		self._messagesView.set_model(self._messagemodel)
+		self._messagesView.set_fixed_height_mode(False)
+
 		self._conversationView = self._messagesView.get_parent()
 		self._conversationViewPort = self._conversationView.get_parent()
 		self._scrollWindow = self._conversationViewPort.get_parent()
 
+		self._targetList = self._widgetTree.get_widget("smsTargetList")
 		self._phoneButton = self._widgetTree.get_widget("phoneTypeSelection")
+		self._phoneButton.connect("clicked", self._on_phone)
 		self._smsEntry = self._widgetTree.get_widget("smsEntry")
+		self._smsEntry.get_buffer().connect("changed", self._on_entry_changed)
 		self._smsEntrySize = None
 
-		self._action = self.ACTION_CANCEL
+		self._contacts = []
 
-		self._numberIndex = -1
-		self._contactDetails = []
+	def add_contact(self, contactDetails, messages = (), defaultIndex = -1):
+		contactNumbers = list(self._to_contact_numbers(contactDetails))
+		assert contactNumbers
+		contactIndex = defaultIndex if defaultIndex != -1 else 0
+		contact = contactNumbers, contactIndex, messages
+		self._contacts.append(contact)
 
-	def run(self, contactDetails, messages = (), parent = None, defaultIndex = -1):
-		entryConnectId = self._smsEntry.get_buffer().connect("changed", self._on_entry_changed)
-		phoneConnectId = self._phoneButton.connect("clicked", self._on_phone)
-		keyConnectId = self._keyPressEventId = self._dialog.connect("key-press-event", self._on_key_press)
-		try:
-			# Setup the phone selection button
-			del self._contactDetails[:]
-			for phoneType, phoneNumber in contactDetails:
-				display = " - ".join((make_pretty(phoneNumber), phoneType))
-				row = (phoneNumber, display)
-				self._contactDetails.append(row)
-			if 0 < len(self._contactDetails):
-				self._numberIndex = defaultIndex if defaultIndex != -1 else 0
-				self._phoneButton.set_label(self._contactDetails[self._numberIndex][1])
-			else:
-				self._numberIndex = -1
-				self._phoneButton.set_label("Error: No Number Available")
+		selector = gtk.Button(contactNumbers[0][1])
+		removeContact = gtk.Button(stock="gtk-delete")
+		row = gtk.HBox()
+		row.pack_start(selector, True, True)
+		row.pack_start(removeContact, False, False)
+		row.show_all()
+		self._targetList.pack_start(row)
+		selector.connect("clicked", self._on_choose_phone_n, row)
+		removeContact.connect("clicked", self._on_remove_phone_n, row)
+		self._update_button_state()
+		self._update_context()
 
-			# Add the column to the messages tree view
-			self._messagemodel.clear()
-			self._messagesView.set_model(self._messagemodel)
-			self._messagesView.set_fixed_height_mode(False)
+		parentSize = self._parent.get_size()
+		self._window.resize(parentSize[0], max(parentSize[1]-10, 100))
+		self._window.show()
+		self._window.present()
 
-			textrenderer = gtk.CellRendererText()
-			textrenderer.set_property("wrap-mode", pango.WRAP_WORD)
-			textrenderer.set_property("wrap-width", 450)
-			messageColumn = gtk.TreeViewColumn("")
-			messageColumn.pack_start(textrenderer, expand=True)
-			messageColumn.add_attribute(textrenderer, "markup", 0)
-			messageColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-			self._messagesView.append_column(messageColumn)
-			self._messagesView.set_headers_visible(False)
+		self._smsEntry.grab_focus()
+		dx = self._conversationView.get_allocation().height - self._conversationViewPort.get_allocation().height
+		dx = max(dx, 0)
+		adjustment = self._scrollWindow.get_vadjustment()
+		adjustment.value = dx
 
-			if messages:
-				for message in messages:
-					row = (message, )
-					self._messagemodel.append(row)
-				self._messagesView.show()
-				self._scrollWindow.show()
-				messagesSelection = self._messagesView.get_selection()
-				messagesSelection.select_path((len(messages)-1, ))
-			else:
-				self._messagesView.hide()
-				self._scrollWindow.hide()
+	def clear(self):
+		del self._contacts[:]
 
-			self._smsEntry.get_buffer().set_text("")
-			self._update_letter_count()
+		for contactNumberSelector in list(self._targetList.get_children()):
+			self._targetList.remove(contactNumberSelector)
+		self._smsEntry.get_buffer().set_text("")
+		self._update_letter_count()
+		self._update_context()
 
-			if parent is not None:
-				self._dialog.set_transient_for(parent)
-				parentSize = parent.get_size()
-				self._dialog.resize(parentSize[0], max(parentSize[1]-10, 100))
+	def fullscreen(self):
+		self._window.fullscreen()
 
-			# Run
-			try:
-				self._dialog.show_all()
-				self._smsEntry.grab_focus()
-				adjustment = self._scrollWindow.get_vadjustment()
-				dx = self._conversationView.get_allocation().height - self._conversationViewPort.get_allocation().height
-				dx = max(dx, 0)
-				adjustment.value = dx
+	def unfullscreen(self):
+		self._window.unfullscreen()
 
-				if 1 < len(self._contactDetails):
-					if defaultIndex == -1:
-						self._request_number()
-					self._phoneButton.set_sensitive(True)
-				else:
-					self._phoneButton.set_sensitive(False)
+	def _remove_contact(self, contactIndex):
+		del self._contacts[contactIndex]
 
-				userResponse = self._dialog.run()
-			finally:
-				self._dialog.hide_all()
+		contactNumberSelector = list(self._targetList.get_children())[contactIndex]
+		self._targetList.remove(contactNumberSelector)
+		self._update_button_state()
+		self._update_context()
 
-			# Process the users response
-			if userResponse == gtk.RESPONSE_OK and 0 <= self._numberIndex:
-				phoneNumber = self._contactDetails[self._numberIndex][0]
-				phoneNumber = make_ugly(phoneNumber)
-			else:
-				phoneNumber = ""
-			if not phoneNumber:
-				self._action = self.ACTION_CANCEL
-			if self._action == self.ACTION_SEND_SMS:
-				entryBuffer = self._smsEntry.get_buffer()
-				enteredMessage = entryBuffer.get_text(entryBuffer.get_start_iter(), entryBuffer.get_end_iter())
-				enteredMessage = enteredMessage[0:self.MAX_CHAR].strip()
-				if not enteredMessage:
-					phoneNumber = ""
-					self._action = self.ACTION_CANCEL
-			else:
-				enteredMessage = ""
-
-			self._messagesView.remove_column(messageColumn)
-			self._messagesView.set_model(None)
-
-			return self._action, phoneNumber, enteredMessage
-		finally:
-			self._smsEntry.get_buffer().disconnect(entryConnectId)
-			self._phoneButton.disconnect(phoneConnectId)
-			self._keyPressEventId = self._dialog.disconnect(keyConnectId)
-
-	def _update_letter_count(self, *args):
+	def _update_letter_count(self):
 		if self._smsEntrySize is None:
 			self._smsEntrySize = self._smsEntry.size_request()
 		else:
 			self._smsEntry.set_size_request(*self._smsEntrySize)
 		entryLength = self._smsEntry.get_buffer().get_char_count()
 
-		charsLeft = self.MAX_CHAR - entryLength
-		self._letterCountLabel.set_text(str(charsLeft))
-		if charsLeft < 0 or charsLeft == self.MAX_CHAR:
-			self._smsButton.set_sensitive(False)
+		numTexts, numCharInText = divmod(entryLength, self.MAX_CHAR)
+		if numTexts:
+			self._letterCountLabel.set_text("%s.%s" % (numTexts, numCharInText))
 		else:
-			self._smsButton.set_sensitive(True)
+			self._letterCountLabel.set_text("%s" % (numCharInText, ))
 
-		if entryLength == 0:
-			self._dialButton.set_sensitive(True)
+		self._update_button_state()
+
+	def _update_context(self):
+		self._messagemodel.clear()
+		if len(self._contacts) == 0:
+			self._messagesView.hide()
+			self._targetList.hide()
+			self._phoneButton.hide()
+			self._phoneButton.set_label("Error: You shouldn't see this")
+		elif len(self._contacts) == 1:
+			contactNumbers, index, messages = self._contacts[0]
+			if messages:
+				self._messagesView.show()
+				for message in messages:
+					row = (message, )
+					self._messagemodel.append(row)
+				messagesSelection = self._messagesView.get_selection()
+				messagesSelection.select_path((len(messages)-1, ))
+			else:
+				self._messagesView.hide()
+			self._targetList.hide()
+			self._phoneButton.show()
+			self._phoneButton.set_label(contactNumbers[index][1])
+			if 1 < len(contactNumbers):
+				self._phoneButton.set_sensitive(True)
+			else:
+				self._phoneButton.set_sensitive(False)
+		else:
+			self._messagesView.hide()
+			self._targetList.show()
+			self._phoneButton.hide()
+			self._phoneButton.set_label("Error: You shouldn't see this")
+
+	def _update_button_state(self):
+		if len(self._contacts) == 0:
+			self._dialButton.set_sensitive(False)
+			self._smsButton.set_sensitive(False)
+		elif len(self._contacts) == 1:
+			entryLength = self._smsEntry.get_buffer().get_char_count()
+			if entryLength == 0:
+				self._dialButton.set_sensitive(True)
+				self._smsButton.set_sensitive(False)
+			else:
+				self._dialButton.set_sensitive(False)
+				self._smsButton.set_sensitive(True)
 		else:
 			self._dialButton.set_sensitive(False)
+			self._smsButton.set_sensitive(True)
 
-	def _request_number(self):
+	def _to_contact_numbers(self, contactDetails):
+		for phoneType, phoneNumber in contactDetails:
+			display = " - ".join((make_pretty(phoneNumber), phoneType))
+			yield (phoneNumber, display)
+
+	def _hide(self):
+		self.clear()
+		self._window.hide()
+
+	def _request_number(self, contactIndex):
+		contactNumbers, index, messages = self._contacts[contactIndex]
+		assert 0 <= index, "%r" % index
+
+		index = hildonize.touch_selector(
+			self._window,
+			"Phone Numbers",
+			(description for (number, description) in contactNumbers),
+			index,
+		)
+		self._contacts[contactIndex] = contactNumbers, index, messages
+
+	def send_sms(self, numbers, message):
+		raise NotImplementedError()
+
+	def dial(self, number):
+		raise NotImplementedError()
+
+	def _on_phone(self, *args):
 		try:
-			assert 0 <= self._numberIndex, "%r" % self._numberIndex
+			assert len(self._contacts) == 1
+			self._request_number(0)
 
-			self._numberIndex = hildonize.touch_selector(
-				self._dialog,
-				"Phone Numbers",
-				(description for (number, description) in self._contactDetails),
-				self._numberIndex,
-			)
-			self._phoneButton.set_label(self._contactDetails[self._numberIndex][1])
+			contactNumbers, numberIndex, messages = self._contacts[0]
+			self._phoneButton.set_label(contactNumbers[numberIndex][1])
+			row = list(self._targetList.get_children())[0]
+			phoneButton = list(row.get_children())[0]
+			phoneButton.set_label(contactNumbers[numberIndex][1])
 		except Exception, e:
 			_moduleLogger.exception("%s" % str(e))
 
-	def _on_phone(self, *args):
-		self._request_number()
+	def _on_choose_phone_n(self, button, row):
+		try:
+			assert 1 < len(self._contacts)
+			targetList = list(self._targetList.get_children())
+			index = targetList.index(row)
+			self._request_number(index)
+
+			contactNumbers, numberIndex, messages = self._contacts[0]
+			phoneButton = list(row.get_children())[0]
+			phoneButton.set_label(contactNumbers[numberIndex][1])
+		except Exception, e:
+			_moduleLogger.exception("%s" % str(e))
+
+	def _on_remove_phone_n(self, button, row):
+		try:
+			assert 1 < len(self._contacts)
+			targetList = list(self._targetList.get_children())
+			index = targetList.index(row)
+
+			del self._contacts[index]
+			self._targetList.remove(row)
+			self._update_context()
+			self._update_button_state()
+		except Exception, e:
+			_moduleLogger.exception("%s" % str(e))
 
 	def _on_entry_changed(self, *args):
 		self._update_letter_count()
 
 	def _on_send(self, *args):
-		self._dialog.response(gtk.RESPONSE_OK)
-		self._action = self.ACTION_SEND_SMS
+		assert 0 < len(self._contacts), "%r" % self._contacts
+		phoneNumbers = [
+			make_ugly(contact[0][contact[1]][0])
+			for contact in self._contacts
+		]
+
+		entryBuffer = self._smsEntry.get_buffer()
+		enteredMessage = entryBuffer.get_text(entryBuffer.get_start_iter(), entryBuffer.get_end_iter())
+		enteredMessage = enteredMessage.strip()
+		assert enteredMessage
+		self.send_sms(phoneNumbers, enteredMessage)
+		self._hide()
 
 	def _on_dial(self, *args):
-		self._dialog.response(gtk.RESPONSE_OK)
-		self._action = self.ACTION_DIAL
+		assert len(self._contacts) == 1, "%r" % self._contacts
+		contact = self._contacts[0]
+		contactNumber = contact[0][contact[1]][0]
+		phoneNumber = make_ugly(contactNumber)
+		self.dial(phoneNumber)
+		self._hide()
 
-	def _on_cancel(self, *args):
-		self._dialog.response(gtk.RESPONSE_CANCEL)
-		self._action = self.ACTION_CANCEL
+	def _on_delete(self, *args):
+		self._window.emit_stop_by_name("delete-event")
+		self._hide()
+		return True
+
+	def _on_window_state_change(self, widget, event, *args):
+		"""
+		@note Hildon specific
+		"""
+		try:
+			if event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN:
+				self._isFullScreen = True
+			else:
+				self._isFullScreen = False
+		except Exception, e:
+			self._errorDisplay.push_exception()
 
 	def _on_key_press(self, widget, event):
+		RETURN_TYPES = (gtk.keysyms.Return, gtk.keysyms.ISO_Enter, gtk.keysyms.KP_Enter)
 		try:
-			if event.keyval == ord("c") and event.get_state() & gtk.gdk.CONTROL_MASK:
+			if (
+				event.keyval == gtk.keysyms.F6 or
+				event.keyval in RETURN_TYPES and event.get_state() & gtk.gdk.CONTROL_MASK
+			):
+				if self._isFullScreen:
+					self._window.unfullscreen()
+				else:
+					self._window.fullscreen()
+			elif event.keyval == ord("c") and event.get_state() & gtk.gdk.CONTROL_MASK:
 				message = "\n".join(
 					messagePart[0]
 					for messagePart in self._messagemodel
 				)
 				self._clipboard.set_text(str(message))
+			elif (
+				event.keyval == gtk.keysyms.w and
+				event.get_state() & gtk.gdk.CONTROL_MASK
+			):
+				self._hide()
+			elif (
+				event.keyval == gtk.keysyms.q and
+				event.get_state() & gtk.gdk.CONTROL_MASK
+			):
+				self._parent.destroy()
 		except Exception, e:
 			_moduleLogger.exception(str(e))
 
@@ -374,12 +517,11 @@ class Dialpad(object):
 	def __init__(self, widgetTree, errorDisplay):
 		self._clipboard = gtk.clipboard_get()
 		self._errorDisplay = errorDisplay
-		self._smsDialog = SmsEntryDialog(widgetTree)
 
 		self._numberdisplay = widgetTree.get_widget("numberdisplay")
-		self._smsButton = widgetTree.get_widget("sms")
-		self._dialButton = widgetTree.get_widget("dial")
+		self._okButton = widgetTree.get_widget("dialpadOk")
 		self._backButton = widgetTree.get_widget("back")
+		self._plusButton = widgetTree.get_widget("plus")
 		self._phonenumber = ""
 		self._prettynumber = ""
 
@@ -387,8 +529,8 @@ class Dialpad(object):
 			"on_digit_clicked": self._on_digit_clicked,
 		}
 		widgetTree.signal_autoconnect(callbackMapping)
-		self._dialButton.connect("clicked", self._on_dial_clicked)
-		self._smsButton.connect("clicked", self._on_sms_clicked)
+		self._okButton.connect("clicked", self._on_ok_clicked)
+		self._plusButton.connect("clicked", self._on_plus)
 
 		self._originalLabel = self._backButton.get_label()
 		self._backTapHandler = gtk_toolbox.TapOrHold(self._backButton)
@@ -401,7 +543,7 @@ class Dialpad(object):
 		self._keyPressEventId = 0
 
 	def enable(self):
-		self._dialButton.grab_focus()
+		self._okButton.grab_focus()
 		self._backTapHandler.enable()
 		self._keyPressEventId = self._window.connect("key-press-event", self._on_key_press)
 
@@ -411,7 +553,7 @@ class Dialpad(object):
 		self._reset_back_button()
 		self._backTapHandler.disable()
 
-	def number_selected(self, action, number, message):
+	def add_contact(self, *args, **kwds):
 		"""
 		@note Actual dial function is patched in later
 		"""
@@ -456,23 +598,13 @@ class Dialpad(object):
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
-	def _on_sms_clicked(self, widget):
+	def _on_ok_clicked(self, widget):
 		try:
 			phoneNumber = self.get_number()
-			action, phoneNumber, message = self._smsDialog.run([("Dialer", phoneNumber)], (), self._window)
-
-			if action == SmsEntryDialog.ACTION_CANCEL:
-				return
-			self.number_selected(action, phoneNumber, message)
-		except Exception, e:
-			self._errorDisplay.push_exception()
-
-	def _on_dial_clicked(self, widget):
-		try:
-			action = SmsEntryDialog.ACTION_DIAL
-			phoneNumber = self.get_number()
-			message = ""
-			self.number_selected(action, phoneNumber, message)
+			self.add_contact(
+				[("Dialer", phoneNumber)], ()
+			)
+			self.set_number("")
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
@@ -482,6 +614,12 @@ class Dialpad(object):
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
+	def _on_plus(self, *args):
+		try:
+			self.set_number(self._phonenumber + "+")
+		except Exception, e:
+			self._errorDisplay.push_exception()
+
 	def _on_backspace(self, taps):
 		try:
 			self.set_number(self._phonenumber[:-taps])
@@ -861,7 +999,6 @@ class CallHistoryView(object):
 		self._nameColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
 		self._window = gtk_toolbox.find_parent_window(self._historyview)
-		self._phoneTypeSelector = SmsEntryDialog(widgetTree)
 
 		self._historyFilterSelector = widgetTree.get_widget("historyFilterSelector")
 		self._historyFilterSelector.connect("clicked", self._on_history_filter_clicked)
@@ -901,7 +1038,7 @@ class CallHistoryView(object):
 		self._historyview.remove_column(self._numberColumn)
 		self._historyview.set_model(None)
 
-	def number_selected(self, action, number, message):
+	def add_contact(self, *args, **kwds):
 		"""
 		@note Actual dial function is patched in later
 		"""
@@ -1036,17 +1173,11 @@ class CallHistoryView(object):
 				contactPhoneNumbers = [("Phone", number)]
 				defaultIndex = -1
 
-			action, phoneNumber, message = self._phoneTypeSelector.run(
+			self.add_contact(
 				contactPhoneNumbers,
 				messages = (description, ),
-				parent = self._window,
 				defaultIndex = defaultIndex,
 			)
-			if action == SmsEntryDialog.ACTION_CANCEL:
-				return
-			assert phoneNumber, "A lack of phone number exists"
-
-			self.number_selected(action, phoneNumber, message)
 			self._historyviewselection.unselect_all()
 		except Exception, e:
 			self._errorDisplay.push_exception()
@@ -1102,7 +1233,6 @@ class MessagesView(object):
 		self._messageColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
 		self._window = gtk_toolbox.find_parent_window(self._messageview)
-		self._phoneTypeSelector = SmsEntryDialog(widgetTree)
 
 		self._messageTypeButton = widgetTree.get_widget("messageTypeButton")
 		self._onMessageTypeClickedId = 0
@@ -1151,7 +1281,7 @@ class MessagesView(object):
 		self._messageview.remove_column(self._messageColumn)
 		self._messageview.set_model(None)
 
-	def number_selected(self, action, number, message):
+	def add_contact(self, *args, **kwds):
 		"""
 		@note Actual dial function is patched in later
 		"""
@@ -1305,17 +1435,11 @@ class MessagesView(object):
 				contactPhoneNumbers = [("Phone", number)]
 				defaultIndex = -1
 
-			action, phoneNumber, message = self._phoneTypeSelector.run(
+			self.add_contact(
 				contactPhoneNumbers,
 				messages = description,
-				parent = self._window,
 				defaultIndex = defaultIndex,
 			)
-			if action == SmsEntryDialog.ACTION_CANCEL:
-				return
-			assert phoneNumber, "A lock of phone number exists"
-
-			self.number_selected(action, phoneNumber, message)
 			self._messageviewselection.unselect_all()
 		except Exception, e:
 			self._errorDisplay.push_exception()
@@ -1406,7 +1530,6 @@ class ContactsView(object):
 		self._onContactsviewRowActivatedId = 0
 		self._onAddressbookButtonChangedId = 0
 		self._window = gtk_toolbox.find_parent_window(self._contactsview)
-		self._phoneTypeSelector = SmsEntryDialog(widgetTree)
 
 		self._updateSink = gtk_toolbox.threaded_stage(
 			gtk_toolbox.comap(
@@ -1458,7 +1581,7 @@ class ContactsView(object):
 		self._contactsview.set_model(None)
 		self._contactsview.remove_column(self._contactColumn)
 
-	def number_selected(self, action, number, message):
+	def add_contact(self, *args, **kwds):
 		"""
 		@note Actual dial function is patched in later
 		"""
@@ -1585,16 +1708,10 @@ class ContactsView(object):
 			if len(contactPhoneNumbers) == 0:
 				return
 
-			action, phoneNumber, message = self._phoneTypeSelector.run(
+			self.add_contact(
 				contactPhoneNumbers,
 				messages = (contactName, ),
-				parent = self._window,
 			)
-			if action == SmsEntryDialog.ACTION_CANCEL:
-				return
-			assert phoneNumber, "A lack of phone number exists"
-
-			self.number_selected(action, phoneNumber, message)
 			self._contactsviewselection.unselect_all()
 		except Exception, e:
 			self._errorDisplay.push_exception()
